@@ -100,9 +100,6 @@ class H11Protocol:
     async def initiate(self) -> None:
         pass
 
-    async def send_task(self) -> None:
-        pass
-
     async def handle(self, event: Event) -> None:
         if isinstance(event, RawData):
             self.connection.receive_data(event.data)
@@ -158,6 +155,14 @@ class H11Protocol:
                 if isinstance(event, h11.Request):
                     await self._check_protocol(event)
                     await self._create_stream(event)
+                elif event is h11.PAUSED:
+                    await self.send(Updated())
+                    await self.can_read.clear()
+                    await self.can_read.wait()
+                elif isinstance(event, h11.ConnectionClosed) or event is h11.NEED_DATA:
+                    break
+                elif self.stream is None:
+                    break
                 elif isinstance(event, h11.Data):
                     await self.stream.handle(Body(stream_id=STREAM_ID, data=event.data))
                 elif isinstance(event, h11.EndOfMessage):
@@ -165,12 +170,6 @@ class H11Protocol:
                 elif isinstance(event, Data):
                     # WebSocket pass through
                     await self.stream.handle(event)
-                elif event is h11.PAUSED:
-                    await self.send(Updated())
-                    await self.can_read.clear()
-                    await self.can_read.wait()
-                elif isinstance(event, h11.ConnectionClosed) or event is h11.NEED_DATA:
-                    break
 
     async def _create_stream(self, request: h11.Request) -> None:
         upgrade_value = ""
@@ -282,7 +281,8 @@ class H11Protocol:
             await self._send_h11_event(
                 h11.InformationalResponse(
                     status_code=101,
-                    headers=[(b"upgrade", b"h2c")] + self.config.response_headers("h11"),
+                    headers=self.config.response_headers("h11")
+                    + [(b"connection", b"upgrade"), (b"upgrade", b"h2c")],
                 )
             )
             raise H2CProtocolRequired(self.connection.trailing_data[0], event)
